@@ -32,9 +32,10 @@ sys.path.insert(0, top_dir)
 from taskflow.conductors import backends as conductor_backends
 from taskflow import engines
 from taskflow.jobs import backends as job_backends
+from taskflow import logging as taskflow_logging
 from taskflow.patterns import linear_flow as lf
 from taskflow.persistence import backends as persistence_backends
-from taskflow.persistence import logbook
+from taskflow.persistence import models
 from taskflow import task
 from taskflow.types import timing
 
@@ -109,7 +110,7 @@ def make_bottles(count):
 
 
 def run_conductor(only_run_once=False):
-    # This continuously consumers until its stopped via ctrl-c or other
+    # This continuously runs consumers until its stopped via ctrl-c or other
     # kill signal...
     event_watches = {}
 
@@ -175,9 +176,9 @@ def run_poster():
             # unit of work we want to complete and the factory that
             # can be called to create the tasks that the work unit needs
             # to be done.
-            lb = logbook.LogBook("post-from-%s" % my_name)
-            fd = logbook.FlowDetail("song-from-%s" % my_name,
-                                    uuidutils.generate_uuid())
+            lb = models.LogBook("post-from-%s" % my_name)
+            fd = models.FlowDetail("song-from-%s" % my_name,
+                                   uuidutils.generate_uuid())
             lb.add(fd)
             with contextlib.closing(persist_backend.get_connection()) as conn:
                 conn.save_logbook(lb)
@@ -220,14 +221,15 @@ def check_for_zookeeper(timeout=1):
 
 
 def main():
-    logging.basicConfig(level=logging.ERROR)
     if not check_for_zookeeper():
         return
     if len(sys.argv) == 1:
         main_local()
     elif sys.argv[1] in ('p', 'c'):
         if sys.argv[-1] == "v":
-            logging.basicConfig(level=5)
+            logging.basicConfig(level=taskflow_logging.TRACE)
+        else:
+            logging.basicConfig(level=logging.ERROR)
         if sys.argv[1] == 'p':
             run_poster()
         else:
